Public Class Form1
    ' The polygon's points.
    Private m_Points() As PointF
    Private m_NumPoints As Integer

    ' The points that have been used in test edges.
    Private m_EdgeChecked() As Boolean

    ' The four caliper control points. They start:
    '       m_ControlPoints(0)      Left edge       xmin
    '       m_ControlPoints(1)      Bottom edge     ymax
    '       m_ControlPoints(2)      Right edge      xmax
    '       m_ControlPoints(3)      Top edge        ymin
    Private m_ControlPoints(3) As Integer

    ' The line from this point to the next one forms
    ' one side of the rectangle.
    Private m_CurrentControlPoint As Integer = -1

    ' The area of the current bounding rectangle.
    Private m_CurrentArea As Single = Single.MaxValue
    Private m_BestArea As Single = Single.MaxValue
    Private m_BestRectangle() As PointF = Nothing

    ' The number of bounding rectangles we have examined.
    Private m_RectanglesExamined As Integer = 0

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Make some points.
        m_Points = New PointF() { _
            New PointF(66, 196), _
            New PointF(70, 210), _
            New PointF(93, 222), _
            New PointF(132, 232), _
            New PointF(195, 237), _
            New PointF(254, 172), _
            New PointF(185, 51), _
            New PointF(87, 92) _
        }
        m_Points = New PointF() { _
            New PointF(67, 201), _
            New PointF(104, 250), _
            New PointF(140, 268), _
            New PointF(183, 272), _
            New PointF(226, 253), _
            New PointF(262, 211), _
            New PointF(279, 149), _
            New PointF(270, 95), _
            New PointF(250, 62), _
            New PointF(217, 37), _
            New PointF(172, 29), _
            New PointF(131, 35), _
            New PointF(94, 57), _
            New PointF(71, 86), _
            New PointF(60, 119), _
            New PointF(59, 156) _
        }
        m_Points = New PointF() { _
            New PointF(45, 148), _
            New PointF(41, 188), _
            New PointF(50, 217), _
            New PointF(67, 244), _
            New PointF(98, 258), _
            New PointF(130, 252), _
            New PointF(186, 233), _
            New PointF(235, 206), _
            New PointF(273, 162), _
            New PointF(285, 117), _
            New PointF(282, 70), _
            New PointF(264, 45), _
            New PointF(234, 28), _
            New PointF(188, 36), _
            New PointF(135, 56), _
            New PointF(89, 93), _
            New PointF(62, 120) _
        }

        m_NumPoints = m_Points.Length

        ' Reset to get ready.
        btnReset_Click(Nothing, Nothing)
    End Sub

    Private Sub picCanvas_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles picCanvas.Paint
        ' See if we have started yet.
        Using box_pen As New Pen(Color.Lime, 3)
            If m_CurrentControlPoint = -1 Then
                ' Draw the initial bounding rectangle.
                DrawBoundingRectangle(e.Graphics, box_pen, _
                    m_Points(m_ControlPoints(0)).X, m_Points(m_ControlPoints(0)).Y, 0, 1, _
                    m_Points(m_ControlPoints(1)).X, m_Points(m_ControlPoints(1)).Y, 1, 0, _
                    m_Points(m_ControlPoints(2)).X, m_Points(m_ControlPoints(2)).Y, 0, -1, _
                    m_Points(m_ControlPoints(3)).X, m_Points(m_ControlPoints(3)).Y, -1, 0)
            Else
                ' See which point has the current edge.
                Dim i1 As Integer = m_ControlPoints(m_CurrentControlPoint)
                Dim i2 As Integer = (i1 + 1) Mod m_NumPoints
                Dim dx As Single = m_Points(i2).X - m_Points(i1).X
                Dim dy As Single = m_Points(i2).Y - m_Points(i1).Y

                ' Make dx and dy work for the first line.
                Select Case m_CurrentControlPoint
                    Case 0  ' Nothing to do.
                    Case 1  ' dx = -dy, dy = dx
                        Dim temp As Single = dx
                        dx = -dy
                        dy = temp
                    Case 2  ' dx = -dx, dy = -dy
                        dx = -dx
                        dy = -dy
                    Case 3  ' dx = dy, dy = -dx
                        Dim temp As Single = dx
                        dx = dy
                        dy = -temp
                End Select

                ' Draw the bounding rectangle.
                DrawBoundingRectangle(e.Graphics, box_pen, _
                    m_Points(m_ControlPoints(0)).X, m_Points(m_ControlPoints(0)).Y, dx, dy, _
                    m_Points(m_ControlPoints(1)).X, m_Points(m_ControlPoints(1)).Y, dy, -dx, _
                    m_Points(m_ControlPoints(2)).X, m_Points(m_ControlPoints(2)).Y, -dx, -dy, _
                    m_Points(m_ControlPoints(3)).X, m_Points(m_ControlPoints(3)).Y, -dy, dx)
            End If
        End Using

        ' Draw the best bounding rectangle.
        e.Graphics.DrawPolygon(Pens.Green, m_BestRectangle)

        ' Draw the polygon and its points.
        Using polygon_pen As New Pen(Color.Black, 5)
            e.Graphics.DrawPolygon(polygon_pen, m_Points)
        End Using

        ' Mark the polygon points.
        Using sf = New StringFormat
            sf.Alignment = StringAlignment.Far
            sf.LineAlignment = StringAlignment.Far
            For i As Integer = 0 To m_NumPoints - 1
                With m_Points(i)
                    e.Graphics.FillRectangle(Brushes.White, .X - 3, .Y - 3, 6, 6)
                    e.Graphics.DrawRectangle(Pens.Black, .X - 3, .Y - 3, 6, 6)
                    e.Graphics.DrawString(i, Me.Font, Brushes.Black, .X, .Y, sf)
                End With
            Next i

            If True Then
                ' Draw the edges that have been checked.
                For i As Integer = 0 To m_NumPoints - 1
                    If m_EdgeChecked(i) Then
                        Dim pt2 As Integer = (i + 1) Mod m_NumPoints
                        Using blue_pen As New Pen(Color.Cyan, 3)
                            e.Graphics.DrawLine(blue_pen, m_Points(i), m_Points(pt2))
                        End Using
                    End If
                Next i

                ' Mark the control points.
                sf.Alignment = StringAlignment.Near
                sf.LineAlignment = StringAlignment.Near
                For i As Integer = 0 To 3
                    With m_Points(m_ControlPoints(i))
                        e.Graphics.FillEllipse(Brushes.Lime, .X - 3, .Y - 3, 6, 6)
                        e.Graphics.DrawString(i, Me.Font, Brushes.Red, .X, .Y, sf)
                    End With
                Next i

                ' Mark the selected control point's edge.
                If m_CurrentControlPoint >= 0 Then
                    Dim pt1 As Integer = m_ControlPoints(m_CurrentControlPoint)
                    Dim pt2 As Integer = (pt1 + 1) Mod m_NumPoints
                    Using red_pen As New Pen(Color.Red, 3)
                        e.Graphics.DrawLine(red_pen, m_Points(pt1), m_Points(pt2))
                    End Using
                End If
            End If
        End Using
    End Sub

    Private Sub DrawBoundingRectangle(ByVal gr As Graphics, ByVal box_pen As Pen, _
        ByVal px0 As Single, ByVal py0 As Single, ByVal dx0 As Single, ByVal dy0 As Single, _
        ByVal px1 As Single, ByVal py1 As Single, ByVal dx1 As Single, ByVal dy1 As Single, _
        ByVal px2 As Single, ByVal py2 As Single, ByVal dx2 As Single, ByVal dy2 As Single, _
        ByVal px3 As Single, ByVal py3 As Single, ByVal dx3 As Single, ByVal dy3 As Single)

        ' Find the points of intersection.
        Dim pts(3) As PointF
        FindIntersection(px0, py0, px0 + dx0, py0 + dy0, px1, py1, px1 + dx1, py1 + dy1, pts(0).X, pts(0).Y)
        FindIntersection(px1, py1, px1 + dx1, py1 + dy1, px2, py2, px2 + dx2, py2 + dy2, pts(1).X, pts(1).Y)
        FindIntersection(px2, py2, px2 + dx2, py2 + dy2, px3, py3, px3 + dx3, py3 + dy3, pts(2).X, pts(2).Y)
        FindIntersection(px3, py3, px3 + dx3, py3 + dy3, px0, py0, px0 + dx0, py0 + dy0, pts(3).X, pts(3).Y)

        ' Draw the bounding rectangle.
        gr.DrawPolygon(box_pen, pts)

        ' See if this is the best bounding rectangle do far.
        ' Get the area of the bounding rectangle.
        Dim vx0 As Single = pts(0).X - pts(1).X
        Dim vy0 As Single = pts(0).Y - pts(1).Y
        Dim len0 As Single = Math.Sqrt(vx0 * vx0 + vy0 * vy0)

        Dim vx1 As Single = pts(1).X - pts(2).X
        Dim vy1 As Single = pts(1).Y - pts(2).Y
        Dim len1 As Single = Math.Sqrt(vx1 * vx1 + vy1 * vy1)

        m_CurrentArea = len0 * len1
        If m_CurrentArea < m_BestArea Then
            m_BestArea = m_CurrentArea
            m_BestRectangle = pts
        End If

        lblCurrentArea.Text = m_CurrentArea.ToString("0.00")
        lblBestArea.Text = m_BestArea.ToString("0.00")
    End Sub

    ' Get ready to start over.
    Private Sub btnReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnReset.Click
        Dim minx As Integer = m_Points(0).X
        Dim maxx As Integer = minx
        Dim miny As Integer = m_Points(0).Y
        Dim maxy As Integer = miny
        Dim minxi As Integer = 0
        Dim maxxi As Integer = 0
        Dim minyi As Integer = 0
        Dim maxyi As Integer = 0

        For i As Integer = 1 To m_NumPoints - 1
            If minx > m_Points(i).X Then
                minx = m_Points(i).X
                minxi = i
            End If
            If maxx < m_Points(i).X Then
                maxx = m_Points(i).X
                maxxi = i
            End If
            If miny > m_Points(i).Y Then
                miny = m_Points(i).Y
                minyi = i
            End If
            If maxy < m_Points(i).Y Then
                maxy = m_Points(i).Y
                maxyi = i
            End If
        Next i

        m_ControlPoints(0) = minxi
        m_ControlPoints(1) = maxyi
        m_ControlPoints(2) = maxxi
        m_ControlPoints(3) = minyi
        m_CurrentControlPoint = -1

        ' We have not yet examined any bounding rectangles.
        m_RectanglesExamined = 0
        btnStep.Enabled = True

        ' Reset the current and best bounding rectangle.
        m_CurrentArea = Single.MaxValue
        m_BestArea = Single.MaxValue
        m_BestRectangle = Nothing

        ' So far we have not checked any edges.
        ReDim m_EdgeChecked(m_NumPoints - 1)
        For i As Integer = 0 To m_NumPoints - 1
            m_EdgeChecked(i) = False
        Next i

        ' Redraw.
        picCanvas.Invalidate()
    End Sub

    Private Sub DrawCaliperLine(ByVal gr As Graphics, ByVal x As Single, ByVal y As Single, ByVal dx As Single, ByVal dy As Single)
        Dim length As Single = Math.Sqrt(dx * dx + dy * dy)
        dx /= length
        dy /= length
        gr.DrawLine(Pens.Blue, x - 20 * dx, y - 20 * dy, x + 100 * dx, y + 100 * dy)
    End Sub

    ' Check the next possible edge.
    Private Sub btnStep_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStep.Click
        ' Increment the current control point.
        ' This says we are done with using this edge.
        If m_CurrentControlPoint >= 0 Then
            m_ControlPoints(m_CurrentControlPoint) = (m_ControlPoints(m_CurrentControlPoint) + 1) Mod m_NumPoints
        End If

        ' Find the next point on an edge.
        Dim xmindx, xmindy As Single
        Dim ymaxdx, ymaxdy As Single
        Dim xmaxdx, xmaxdy As Single
        Dim ymindx, ymindy As Single
        FindDxDy(xmindx, xmindy, m_ControlPoints(0))
        FindDxDy(ymaxdx, ymaxdy, m_ControlPoints(1))
        FindDxDy(xmaxdx, xmaxdy, m_ControlPoints(2))
        FindDxDy(ymindx, ymindy, m_ControlPoints(3))

        ' Switch so we can look for the smallest opposite/adjacent ratio.
        Dim xminopp As Single = xmindx
        Dim xminadj As Single = xmindy
        Dim ymaxopp As Single = -ymaxdy
        Dim ymaxadj As Single = ymaxdx
        Dim xmaxopp As Single = -xmaxdx
        Dim xmaxadj As Single = -xmaxdy
        Dim yminopp As Single = ymindy
        Dim yminadj As Single = -ymindx

        ' Pick initial values that will make every point an improvement.
        Dim bestopp As Single = 1
        Dim bestadj As Single = 0
        Dim best_control_point As Integer = -1

        If xminopp >= 0 AndAlso xminadj >= 0 Then
            If xminopp * bestadj < bestopp * xminadj Then
                bestopp = xminopp
                bestadj = xminadj
                best_control_point = 0
            End If
        End If
        If ymaxopp >= 0 AndAlso ymaxadj >= 0 Then
            If ymaxopp * bestadj < bestopp * ymaxadj Then
                bestopp = ymaxopp
                bestadj = ymaxadj
                best_control_point = 1
            End If
        End If
        If xmaxopp >= 0 AndAlso xmaxadj >= 0 Then
            If xmaxopp * bestadj < bestopp * xmaxadj Then
                bestopp = xmaxopp
                bestadj = xmaxadj
                best_control_point = 2
            End If
        End If
        If yminopp >= 0 AndAlso yminadj >= 0 Then
            If yminopp * bestadj < bestopp * yminadj Then
                bestopp = yminopp
                bestadj = yminadj
                best_control_point = 3
            End If
        End If

        ' Make sure we found a usable edge.
        If best_control_point < 0 Then Stop

        ' Use the new best control point.
        m_CurrentControlPoint = best_control_point

        ' Remember that we have checked this edge.
        m_EdgeChecked(m_ControlPoints(m_CurrentControlPoint)) = True

        ' See if we have checked all possible bounding rectangles.
        m_RectanglesExamined += 1
        If m_RectanglesExamined >= m_Points.Length Then
            btnStep.Enabled = False
        End If

        picCanvas.Invalidate()
    End Sub

    ' Find the slope of the edge from point i to point i+1.
    Private Sub FindDxDy(ByRef dx As Single, ByRef dy As Single, ByVal i As Integer)
        Dim i2 As Integer = (i + 1) Mod m_NumPoints
        dx = m_Points(i2).X - m_Points(i).X
        dy = m_Points(i2).Y - m_Points(i).Y
    End Sub

    ' Find the point of intersection between two lines.
    Private Function FindIntersection(ByVal X1 As Single, ByVal Y1 As Single, ByVal X2 As Single, ByVal Y2 As Single, ByVal A1 As Single, ByVal B1 As Single, ByVal A2 As Single, ByVal B2 As Single, ByRef intersect_x As Single, ByRef intersect_y As Single) As Boolean
        Dim dx As Single = X2 - X1
        Dim dy As Single = Y2 - Y1
        Dim da As Single = A2 - A1
        Dim db As Single = B2 - B1
        Dim s, t As Single

        ' If the segments are parallel, return False.
        If Math.Abs(da * dy - db * dx) < 0.001 Then Return False

        ' Find the point of intersection.
        s = (dx * (B1 - Y1) + dy * (X1 - A1)) / (da * dy - db * dx)
        t = (da * (Y1 - B1) + db * (A1 - X1)) / (db * dx - da * dy)
        intersect_x = X1 + t * dx
        intersect_y = Y1 + t * dy
        Return True
    End Function

    Private Sub picCanvas_MouseClick(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseClick
        Debug.WriteLine("            New PointF(" & e.X & ", " & e.Y & "), _")
    End Sub
End Class
